#  Copyright © 2025 Bentley Systems, Incorporated
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#      http://www.apache.org/licenses/LICENSE-2.0
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


from enum import Enum

from evo.common.data import OrderByOperatorEnum


def parse_order_by(order_by: dict[Enum | str, OrderByOperatorEnum | str] | None) -> str | None:
    if order_by is None:
        return None
    parts = []
    for field, op in order_by.items():
        if isinstance(field, Enum):
            field = field.value
        if isinstance(op, Enum):
            op = op.value
        parts.append(f"{op}:{field}")
    return ",".join(parts)
